/***************************************************************************
 *   Copyright (C) 2007 by Dr. Feynman                                     *
 *   dr@feynman.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __FEYNMAN_H__
#define __FEYNMAN_H__

#include <sys/time.h>

#ifdef WIN32
#include <windows.h>
#else
#include <pthread.h>
#endif

#ifdef USING_OPENSSL
#include <openssl/ssl.h>
#endif

#ifdef USING_CYASSL
#include <cyassl/ssl.h>
#endif

//Messages: 0x60 --> 0x8F
#define FEYNMAN_MSG_LOGIN               0X60
#define FEYNMAN_MSG_LOGIN_ACK           0X61
#define FEYNMAN_MSG_ADD_CARD            0x62
#define FEYNMAN_MSG_DEL_CARD            0x63
#define FEYNMAN_MSG_SIDS                0x64
#define FEYNMAN_MSG_ECM                 0X65
#define FEYNMAN_MSG_ECM_ACK             0X66
#define FEYNMAN_MSG_EMM                 0X67
#define FEYNMAN_MSG_EMM_ACK             0X68


//Data: 0x20 --> 0x5F
#define FEYNMAN_DATA_CARD_ID            0X20
#define FEYNMAN_DATA_CA_ID              0X21
#define FEYNMAN_DATA_PROVIDER_ID        0X22
#define FEYNMAN_DATA_SID                0X23
#define FEYNMAN_DATA_ECM                0X24
#define FEYNMAN_DATA_DCW                0X25
#define FEYNMAN_DATA_EMM                0X26
#define FEYNMAN_DATA_EMM_TYPE           0X27
#define FEYNMAN_DATA_EMM_ADDRESS        0X28
#define FEYNMAN_DATA_GA                 0X29
#define FEYNMAN_DATA_SA                 0X2A
#define FEYNMAN_DATA_UA                 0X2B
#define FEYNMAN_DATA_SESSION_ID         0X2C
#define FEYNMAN_DATA_USERNAME           0X2D
#define FEYNMAN_DATA_PASSWORD           0X2E
#define FEYNMAN_DATA_ACCESS_MODE        0X2F

//Access responses
#define FEYNMAN_ACCESS_MODE_NOACCESS    0X00
#define FEYNMAN_ACCESS_MODE_ACCESSOK    0X01
#define FEYNMAN_ACCESS_MODE_ECM         0X02
#define FEYNMAN_ACCESS_MODE_EMM         0X04
#define FEYNMAN_ACCESS_MODE_GA          0X08
#define FEYNMAN_ACCESS_MODE_SA          0X10
#define FEYNMAN_ACCESS_MODE_UA          0X20
#define FEYNMAN_ACCESS_MODE_IGNORE_SID  0X40

//Address types
#define FEYNMAN_ADDRESS_TYPE_GA         FEYNMAN_ACCESS_MODE_GA
#define FEYNMAN_ADDRESS_TYPE_SA         FEYNMAN_ACCESS_MODE_SA
#define FEYNMAN_ADDRESS_TYPE_UA         FEYNMAN_ACCESS_MODE_UA

//Sid modes
#define FEYNMAN_SIDMODE_BLOCK           0X00
#define FEYNMAN_SIDMODE_ALLOW           0X01
#define FEYNMAN_SIDMODE_UNKNOWN         0X02

//ECM status
#define FEYNMAN_ECMSTATUS_WAITING       0x00
#define FEYNMAN_ECMSTATUS_DECRYPT_OK    0x01
#define FEYNMAN_ECMSTATUS_DECRYPT_NOK   0x02
#define FEYNMAN_ECMSTATUS_IDLE          0x03
#define FEYNMAN_ECMSTATUS_ERROR         0x04
#define FEYNMAN_ECMSTATUS_POOL_EMPTY    0x05

//Used all over the place ;)
#define FEYNMAN_ERROR                   -1
#define FEYNMAN_STATUS_NOK              0X00
#define FEYNMAN_STATUS_OK               0X01
#define FEYNMAN_BUF_SIZE                0x200

#define FEYNMAN_LOGMODE_NONE            0
#define FEYNMAN_LOGMODE_NORMAL          1
#define FEYNMAN_LOGMODE_VERBOSE         2
#define FEYNMAN_LOGMODE_ALL             4

void dump_array(unsigned char *buf, int len);

typedef struct
{
    unsigned short caid;
    int provid;
    int address;
    int type;
    unsigned char *emm;
    int emm_len;
    int card_id;
    unsigned short session;
    struct timeval time_added;
    void *next;
}t_feynman_emm;

typedef struct
{
    int fd;
#ifdef HAVE_SSL
    SSL *ssl;
#endif
    long bytes_in;
    long bytes_out;
    void *next;
}t_feynman_handle;

typedef struct
{
    t_feynman_handle handle;
    int access_mode;
    char *username;
    char *password;
    int client_id;
    void *server;
    void *next;
}t_feynman_client;

typedef struct
{
    unsigned short caid;
    int provid;
    unsigned short sid;
    unsigned char *ecm;
    int ecm_len;
    int decrypt_status;
    unsigned char *dcw;
    int dcw_len;
    unsigned short session;
    int server_id;
    t_feynman_client *client;
    int card_id;
    struct timeval time_added;
    void *next;
}t_feynman_ecm;

typedef struct
{
    unsigned short sid;
    char mode;
    int card_id;
    void *next;
}t_feynman_sid;

typedef struct
{
    int card_id;
    unsigned short caid;
    int provid;
    int ga;
    int sa;
    int ua;
    t_feynman_sid *sid_db;
    void *next;
}t_feynman_card;

typedef int(t_feynman_dcw_callback)(void *args, t_feynman_ecm *ecm);
typedef int(t_feynman_ecm_callback)(void *args, t_feynman_ecm *ecm);
typedef int(t_feynman_emm_callback)(void *args, t_feynman_emm *emm);

typedef struct
{
    t_feynman_handle handle;
    char logged_in;
    int access_mode;
    int ready;
    char *username;
    char *password;
    char *hostname;
    unsigned short port;
    int id;

    t_feynman_card *card_db;
    t_feynman_ecm *ecm_db;
    t_feynman_emm *emm_db;

#ifdef WIN32
    HANDLE server_lock;
#else
    pthread_mutex_t server_lock;
    pthread_t socket_thread;
    pthread_t ecm_thread;
#endif

    t_feynman_dcw_callback *dcw_callback;
    void *dcw_callback_args;
#ifdef HAVE_SSL
    SSL_CTX *ctx;
#endif
    void *next;
}t_feynman_remote_server;

typedef struct
{
    int ready;
    unsigned short port;
    int id;

    t_feynman_client *client_db;
    t_feynman_card *card_db;
    t_feynman_ecm *ecm_db;
    t_feynman_emm *emm_db;
    t_feynman_handle *handle_db;

#ifdef WIN32
    HANDLE server_lock;
#else
    pthread_mutex_t server_lock;
    pthread_t socket_thread;
#endif

    t_feynman_ecm_callback *ecm_callback;
    t_feynman_emm_callback *emm_callback;
    void *ecm_callback_args;
    void *emm_callback_args;
#ifdef HAVE_SSL
    SSL_CTX *ctx;
#endif
    void *next;
}t_feynman_local_server;

void feynman_compile_time();

///////////////////// For client part /////////////////////
t_feynman_remote_server *feynman_add_remote_server(t_feynman_remote_server *server_db);
t_feynman_remote_server *feynman_get_remote_server(t_feynman_remote_server *server_db, int server_id);
t_feynman_remote_server *feynman_del_remote_server(t_feynman_remote_server *server_db, int server_id);

void feynman_set_remote_server_username(t_feynman_remote_server *server, char *username);
void feynman_set_remote_server_password(t_feynman_remote_server *server, char *password);
void feynman_set_remote_server_hostname(t_feynman_remote_server *server, char *hostname);
void feynman_set_remote_server_port(t_feynman_remote_server *server, int port);
void feynman_set_remote_server_dcw_callback(t_feynman_remote_server *server, t_feynman_dcw_callback callback);
void feynman_set_remote_server_dcw_callback_args(t_feynman_remote_server *server, void *args);

int feynman_init_remote_server(t_feynman_remote_server *server);
#ifdef HAVE_SSL
int feynman_init_remote_server_ssl(t_feynman_remote_server *server, char *cert, char *key);
#endif

int feynman_send_server_ecm(t_feynman_remote_server *server, unsigned short session, int card_id, unsigned short caid, int provid, unsigned short sid, unsigned char *ecm_data, int len);
void feynman_send_server_emm(t_feynman_remote_server *server, unsigned short caid, int provid, int address, unsigned char *emm_data, int len);
int feynman_dcw_dummy_callback(t_feynman_remote_server *server, unsigned short session, unsigned char *dcw, int timeout_ms);

///////////////////// For server part /////////////////////
t_feynman_local_server *feynman_add_local_server(t_feynman_local_server *server_db);
t_feynman_local_server *feynman_del_local_server(t_feynman_local_server *server_db, t_feynman_local_server *server);

void feynman_set_local_server_port(t_feynman_local_server *server, int port);
void feynman_set_local_server_ecm_callback(t_feynman_local_server *server, t_feynman_ecm_callback callback);
void feynman_set_local_server_emm_callback(t_feynman_local_server *server, t_feynman_emm_callback callback);
void feynman_set_local_server_emm_callback_args(t_feynman_local_server *server, void *args);
void feynman_set_local_server_ecm_callback_args(t_feynman_local_server *server, void *args);
void feynman_set_ecm_dcw(t_feynman_ecm *ecm, unsigned char *dcw, int dcw_len);

t_feynman_client *feynman_add_client(t_feynman_local_server *server);
void feynman_del_client(t_feynman_local_server *server, t_feynman_client *client);
void feynman_set_client_username(t_feynman_client *client, char *username);
void feynman_set_client_password(t_feynman_client *client, char *password);
void feynman_set_client_access_mode(t_feynman_client *client, int access_mode);
void feynman_drop_client(t_feynman_local_server *server, t_feynman_handle *handle);

t_feynman_card *feynman_add_local_card(t_feynman_local_server *server, int card_id);
void feynman_del_local_card(t_feynman_local_server *server, t_feynman_card *card);
void feynman_set_local_card_caid(t_feynman_card *card, unsigned short caid);
void feynman_set_local_card_provid(t_feynman_card *card, int provid);
void feynman_set_local_card_ga(t_feynman_card *card, int ga);
void feynman_set_local_card_sa(t_feynman_card *card, int sa);
void feynman_set_local_card_ua(t_feynman_card *card, int ua);

//Will also update the sid mode if its allready on the card
t_feynman_sid *feynman_add_sid(t_feynman_card *card, unsigned short new_sid, char mode);

int feynman_start_local_server(t_feynman_local_server *server);
#ifdef HAVE_SSL
int feynman_start_local_server_ssl(t_feynman_local_server *server, char *cert, char *key);
#endif

//Only to be used with your own socket server. Remember to lock/unlock the server
int feynman_process_client_msg(t_feynman_local_server *server, t_feynman_handle *handle, unsigned char *buf, int len);
void feynman_lock_local_server(t_feynman_local_server *server);
void feynman_unlock_local_server(t_feynman_local_server *server);
t_feynman_handle *feynman_add_handle(t_feynman_local_server *server, int fd);
void feynman_del_handle(t_feynman_local_server *server, t_feynman_handle *handle);
t_feynman_handle *feynman_get_handle(t_feynman_local_server *server, int fd);


///////////////////// Internal stuff /////////////////////

//Server stuff
void feynman_lock_remote_server(t_feynman_remote_server *server);
void feynman_unlock_remote_server(t_feynman_remote_server *server);
int feynman_connect_remote_server(t_feynman_remote_server *server);
void feynman_close_remote_server(t_feynman_remote_server *server);

//Client stuff
t_feynman_client *feynman_get_client(t_feynman_local_server *server, t_feynman_handle *handle);
void feynman_del_clients(t_feynman_local_server *server);

//Card stuff
t_feynman_card *feynman_add_remote_card(t_feynman_remote_server *server, int card_id);
t_feynman_card *feynman_get_remote_card(t_feynman_remote_server *server, int card_id);
t_feynman_card *feynman_get_local_card(t_feynman_local_server *server, int card_id);

void feynman_del_remote_card(t_feynman_remote_server *server, int card_id);
void feynman_del_local_cards(t_feynman_local_server *server);
void feynman_del_remote_cards(t_feynman_remote_server *server);

//Sid stuff
t_feynman_sid *feynman_get_sid(t_feynman_card *card, unsigned short service_id);
void feynman_del_sid(t_feynman_card *card, t_feynman_sid *sid);
void feynman_del_sids(t_feynman_card *card);

//ECM stuff
t_feynman_ecm *feynman_get_local_ecm(t_feynman_local_server *server, int session, int card_id);
t_feynman_ecm *feynman_get_remote_ecm(t_feynman_remote_server *server, int session, int card_id);

t_feynman_ecm *feynman_add_local_ecm(t_feynman_local_server *server);
t_feynman_ecm *feynman_add_remote_ecm(t_feynman_remote_server *server);

void feynman_del_local_ecm(t_feynman_local_server *server, t_feynman_ecm *ecm_to_del);
void feynman_del_remote_ecm(t_feynman_remote_server *server, t_feynman_ecm *ecm_to_del);

void feynman_del_local_ecms(t_feynman_local_server *server);
void feynman_del_remote_ecms(t_feynman_remote_server *server);

void feynman_set_ecm_data(t_feynman_ecm *ecm, unsigned char *ecm_data, int ecm_len);
void feynman_free_ecm(t_feynman_ecm *ecm);

//EMM stuff
t_feynman_emm *feynman_add_remote_emm(t_feynman_remote_server *server);
t_feynman_emm *feynman_add_local_emm(t_feynman_local_server *server);

void feynman_set_emm_data(t_feynman_emm *emm, unsigned char *emm_data, int emm_len);
void feynman_free_emm(t_feynman_emm *emm);

t_feynman_emm *feynman_get_remote_emm(t_feynman_remote_server *server, int session, int card_id);
t_feynman_emm *feynman_get_local_emm(t_feynman_local_server *server, int session, int card_id);

void feynman_del_remote_emm(t_feynman_remote_server *server, t_feynman_emm *emm);
void feynman_del_local_emm(t_feynman_local_server *server, t_feynman_emm *emm);
void feynman_del_remote_emms(t_feynman_remote_server *server);
void feynman_del_local_emms(t_feynman_local_server *server);

//Handle stuff
void feynman_del_handles(t_feynman_local_server *server);

//Tools
int feynman_send_network_message(t_feynman_handle handle, unsigned char *buf, int len);
int feynman_read_network_message(t_feynman_handle handle, unsigned char *buf);

void feynman_get_time_diff(struct timeval * tstart, struct timeval * tend);
int feynman_get_supprted_cards(t_feynman_remote_server *server, int card_db[255], unsigned short caid, int provid, unsigned short sid);

void feynman_logging(int mode, char *fmt,...);
void feynman_logging_set_mode(int mode);

//Messages
int feynman_send_login(t_feynman_remote_server *server);
t_feynman_client *feynman_process_login(t_feynman_local_server *server, unsigned char *buf, int len);
int feynman_send_login_ack(t_feynman_client *client, t_feynman_handle *handle);

int feynman_send_add_card(t_feynman_client *client, t_feynman_card *card);
int feynman_process_add_card(t_feynman_remote_server *server, unsigned char *buf, int len);

int feynman_send_del_card(t_feynman_client *client, t_feynman_card *card);
int feynman_process_del_card(t_feynman_remote_server *server, unsigned char *buf, int len);

int feynman_send_sid(t_feynman_client *client, int card_id, t_feynman_sid *sid);
int feynman_send_sidlist(t_feynman_client *client, t_feynman_card *card);
int feynman_process_sids(t_feynman_remote_server *server, unsigned char *buf, int len);

int feynman_send_ecm(t_feynman_remote_server *server, t_feynman_ecm *ecm);
int feynman_process_ecm(t_feynman_client *client, t_feynman_local_server *server, unsigned char *buf, int len);

int feynman_send_ecm_ack(t_feynman_ecm *ecm);
int feynman_process_ecm_ack(t_feynman_remote_server *server, unsigned char *buf, int len);

int feynman_process_remote_server_msg(t_feynman_remote_server *server, unsigned char *buf, int len);

#endif
