/***************************************************************************
 *   Copyright (C) 2007 by Dr. Feynman                                     *
 *   dr@feynman.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>

#include "feynman.h"

t_feynman_card *feynman_add_remote_card(t_feynman_remote_server *server, int card_id)
{
    if(!server) return NULL;

    t_feynman_card *card = feynman_get_remote_card(server, card_id);
    if(card) return card;

    card = (t_feynman_card *)malloc(sizeof(t_feynman_card));

    card->caid = 0;
    card->provid = 0;
    card->ga = 0;
    card->sa = 0;
    card->ua = 0;
    card->sid_db = NULL;
    card->card_id = card_id;

    card->next = server->card_db;
    server->card_db = card;

    return card;
}

t_feynman_card *feynman_add_local_card(t_feynman_local_server *server, int card_id)
{
    if(!server) return NULL;

    t_feynman_card *card = feynman_get_local_card(server, card_id);
    if(card) return card;

    card = (t_feynman_card *)malloc(sizeof(t_feynman_card));

    card->caid = 0;
    card->provid = 0;
    card->ga = 0;
    card->sa = 0;
    card->ua = 0;
    card->sid_db = NULL;
    card->card_id = card_id;

    card->next = server->card_db;
    server->card_db = card;

    return card;
}

void feynman_del_remote_card(t_feynman_remote_server *server, int card_id)
{
    if(!server) return;
    if(!server->card_db) return;

    t_feynman_card *card = server->card_db;
    t_feynman_card *prev = server->card_db;
    while(card != NULL)
    {
        t_feynman_card *tmp_card = card->next;
        if(card_id == -1 || card_id == card->card_id)
        {
            if(prev != NULL)
            {
                prev->next = card->next;
            }
            if(server->card_db == card)
            {
                server->card_db = card->next;
            }
            feynman_del_sids(card);
            free(card);
            if(card_id != -1) return;
        }
        prev = card;
        card = tmp_card;
    }
}
void feynman_del_remote_cards(t_feynman_remote_server *server)
{
    if(!server) return;
    if(!server->card_db) return;

    t_feynman_card *card = server->card_db;
    t_feynman_card *prev = server->card_db;
    while(card != NULL)
    {
        t_feynman_card *tmp_card = card->next;
        if(1)
        {
            if(prev != NULL)
            {
                prev->next = card->next;
            }
            if(server->card_db == card)
            {
                server->card_db = card->next;
            }
            feynman_del_sids(card);
            free(card);
        }
        prev = card;
        card = tmp_card;
    }
}

t_feynman_card *feynman_get_remote_card(t_feynman_remote_server *server, int card_id)
{
    t_feynman_card *i = NULL;

    if(!server) return NULL;
    if(!server->card_db) return NULL;

    for(i=server->card_db; i!=NULL; i=i->next)
    {
        if(card_id == i->card_id)
        {
            return i;
        }
    }
    return i;
}

t_feynman_card *feynman_get_local_card(t_feynman_local_server *server, int card_id)
{
    t_feynman_card *i = NULL;

    if(!server) return NULL;
    if(!server->card_db) return NULL;

    for(i=server->card_db; i!=NULL; i=i->next)
    {
        if(card_id == i->card_id)
        {
            return i;
        }
    }
    return i;
}

void feynman_set_local_card_caid(t_feynman_card *card, unsigned short caid)
{
    if(!card) return;
    card->caid = caid;
}

void feynman_set_local_card_provid(t_feynman_card *card, int provid)
{
    if(!card) return;
    card->provid = provid;
}

void feynman_set_local_card_ga(t_feynman_card *card, int ga)
{
    if(!card) return;
    card->ga = ga;
}

void feynman_set_local_card_sa(t_feynman_card *card, int sa)
{
    if(!card) return;
    card->sa = sa;
}

void feynman_set_local_card_ua(t_feynman_card *card, int ua)
{
    if(!card) return;
    card->ua = ua;
}

void feynman_del_local_card(t_feynman_local_server *server, t_feynman_card *card_to_del)
{
    if(!server) return;
    if(!server->card_db) return;
	if(!card_to_del) return;
	
    t_feynman_card *card = server->card_db;
    t_feynman_card *prev = server->card_db;
    while(card != NULL)
    {
        t_feynman_card *tmp_card = card->next;
        if(card_to_del == card)
        {
            if(prev != NULL)
            {
                prev->next = card->next;
            }
            if(server->card_db == card)
            {
                server->card_db = card->next;
            }
            feynman_del_sids(card);
            free(card);
            return;
        }
        prev = card;
        card = tmp_card;
    }
}

void feynman_del_local_cards(t_feynman_local_server *server)
{
    if(!server) return;
    if(!server->card_db) return;
	
    t_feynman_card *card = server->card_db;
    t_feynman_card *prev = server->card_db;
    while(card != NULL)
    {
        t_feynman_card *tmp_card = card->next;
        if(1)
        {
            if(prev != NULL)
            {
                prev->next = card->next;
            }
            if(server->card_db == card)
            {
                server->card_db = card->next;
            }
            feynman_del_sids(card);
            free(card);
        }
        prev = card;
        card = tmp_card;
    }
}

int feynman_get_supprted_cards(t_feynman_remote_server *server, int card_db[255], unsigned short caid, int provid, unsigned short sid)
{
    if(!server) return 0;

    t_feynman_card *card = NULL;
    t_feynman_sid *tmp_sid = NULL;

    int blocked = 0;
    int card_count = 0;

    //look for cards WITH the sid
    //We do this twise so that all with sid allow comes first in the list.
    for(card=server->card_db; card!=NULL; card=card->next)
    {
        if(caid == card->caid && provid == card->provid)
        {
            blocked = 1;
            for(tmp_sid=card->sid_db; tmp_sid!=NULL; tmp_sid=tmp_sid->next)
            {
                if(sid == tmp_sid->sid)
                {
                    if(tmp_sid->mode == FEYNMAN_SIDMODE_ALLOW || server->access_mode & FEYNMAN_ACCESS_MODE_IGNORE_SID)
                    {
                        blocked = 0;
                    }
                }
                if(!blocked)
                {
                    card_db[card_count++] = card->card_id;
                    blocked = 1;
                    if(card_count == 254) return card_count;
                }
            }
        }
    }

    //look for cards WIHTOUT the sid
    for(card=server->card_db; card!=NULL; card=card->next)
    {
        if(caid == card->caid && provid == card->provid)
        {
            int got_sid = 0;
            for(tmp_sid=card->sid_db; tmp_sid!=NULL; tmp_sid=tmp_sid->next)
            {
                if(sid == tmp_sid->sid)
                {
                    got_sid = 1;
                }
            }
            if(!got_sid)
            {
                card_db[card_count++] = card->card_id;
                if(card_count == 254) return card_count;
            }
        }
    }
    return card_count;
}
