/***************************************************************************
 *   Copyright (C) 2007 by Dr. Feynman                                     *
 *   dr@feynman.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>
#include <string.h>
#ifndef WIN32
#include <unistd.h>
#define closesocket(x) close(x)
#endif

#include "feynman.h"

 t_feynman_client *feynman_add_client(t_feynman_local_server *server)
{
    if(!server) return NULL;

    t_feynman_client *client;

    client = (t_feynman_client *)malloc(sizeof(t_feynman_client));

    client->handle.fd = 0;
#ifdef HAVE_SSL
    client->handle.ssl = NULL;
#endif
    client->handle.bytes_in = 0;
    client->handle.bytes_out = 0;

    client->access_mode = 0;

    client->username = NULL;
    client->password = NULL;
    client->server = server;

    client->next = server->client_db;
    server->client_db = client;

    if(server->client_db != NULL)
    {
        client->client_id = server->client_db->client_id + 1;
    }
    else
    {
        client->client_id = 1;
    }

    return client;
}

void feynman_free_client(t_feynman_client *client)
{
    if(!client) return;

    if(client->username != NULL)
    {
        free(client->username);
    }
    if(client->password != NULL)
    {
        free(client->password);
    }
    free(client);
}

void feynman_del_client(t_feynman_local_server *server, t_feynman_client *client_to_del)
{
    if(!server) return;
    if(!server->client_db) return;
    if(!client_to_del) return;

    t_feynman_client *client = server->client_db;
    t_feynman_client *prev = server->client_db;
    while(client != NULL)
    {
        t_feynman_client *tmp_client = client->next;
        if(client_to_del == client)
        {
            if(prev != NULL)
            {
                prev->next = client->next;
            }
            if(server->client_db == client)
            {
                server->client_db = client->next;
            }
            feynman_free_client(client);
            return;
        }
        prev = client;
        client = tmp_client;
    }
}

void feynman_del_clients(t_feynman_local_server *server)
{
    if(!server) return;
    if(!server->client_db) return;

    t_feynman_client *client = server->client_db;
    t_feynman_client *prev = server->client_db;
    while(client != NULL)
    {
        t_feynman_client *tmp_client = client->next;
        if(1)
        {
            if(prev != NULL)
            {
                prev->next = client->next;
            }
            if(server->client_db == client)
            {
                server->client_db = client->next;
            }
            feynman_free_client(client);
        }
        prev = client;
        client = tmp_client;
    }
}

t_feynman_client *feynman_get_client(t_feynman_local_server *server, t_feynman_handle *handle)
{
    if(!server) return NULL;
    if(!handle) return NULL;

    t_feynman_client *client;

    for(client=server->client_db; client!=NULL; client=client->next)
    {
        if(client->handle.fd == handle->fd)
        {
            return client;
        }
    }

    return NULL;
}

void feynman_set_client_username(t_feynman_client *client, char *username)
{
    if(!client) return;
    if(!username) return;

    int len = strlen(username);
    client->username = (char *)malloc(len+1);
    memcpy(client->username, username, len);
    client->username[len] = '\0';
}

void feynman_set_client_password(t_feynman_client *client, char *password)
{
    if(!client) return;
    if(!password) return;

    int len = strlen(password);
    client->password = (char *)malloc(len+1);
    memcpy(client->password, password, len);
    client->password[len] = '\0';
}

void feynman_set_client_access_mode(t_feynman_client *client, int access_mode)
{
    if(!client) return;
    client->access_mode = access_mode;
}

void feynman_drop_client(t_feynman_local_server *server, t_feynman_handle *handle)
{
    if(!handle) return;
    if(!server) return;

    t_feynman_client *client = feynman_get_client(server, handle);

    if(client != NULL)
    {
        if(client->handle.fd > 0)
        {
            closesocket(client->handle.fd);
            client->handle.fd = 0;
        }
#ifdef HAVE_SSL
        if(client->handle.ssl > 0)
        {
            SSL_shutdown(client->handle.ssl);
            SSL_free(client->handle.ssl);
            client->handle.ssl = NULL;
        }
#endif
    }
}
