/***************************************************************************
 *   Copyright (C) 2007 by Dr. Feynman                                     *
 *   dr@feynman.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>

#include "feynman.h"

t_feynman_handle *feynman_add_handle(t_feynman_local_server *server, int fd)
{
    if(!server) return NULL;

    t_feynman_handle *handle = feynman_get_handle(server, fd);
    if(handle) return handle;

    handle = (t_feynman_handle *)malloc(sizeof(t_feynman_handle));

    handle->fd = fd;
#ifdef HAVE_SSL
    handle->ssl = NULL;
#endif
    handle->bytes_in = 0;
    handle->bytes_out = 0;

    handle->next = server->handle_db;
    server->handle_db = handle;

    return handle;
}

void feynman_del_handle(t_feynman_local_server *server, t_feynman_handle *handle_to_del)
{
    if(!server) return;
    if(!server->handle_db) return;
    if(!handle_to_del) return;

    t_feynman_handle *handle = server->handle_db;
    t_feynman_handle *prev = server->handle_db;
    while(handle != NULL)
    {
        t_feynman_handle *tmp_handle = handle->next;
        if(handle_to_del == handle)
        {
            if(prev != NULL)
            {
                prev->next = handle->next;
            }
            if(server->handle_db == handle)
            {
                server->handle_db = handle->next;
            }
            free(handle);
            return;
        }
        prev = handle;
        handle = tmp_handle;
    }
}

void feynman_del_handles(t_feynman_local_server *server)
{
    if(!server) return;
    if(!server->handle_db) return;

    t_feynman_handle *handle = server->handle_db;
    t_feynman_handle *prev = server->handle_db;
    while(handle != NULL)
    {
        t_feynman_handle *tmp_handle = handle->next;
        if(1)
        {
            if(prev != NULL)
            {
                prev->next = handle->next;
            }
            if(server->handle_db == handle)
            {
                server->handle_db = handle->next;
            }
            free(handle);
        }
        prev = handle;
        handle = tmp_handle;
    }
}

t_feynman_handle *feynman_get_handle(t_feynman_local_server *server, int fd)
{
    if(!server) return NULL;

    t_feynman_handle *handle;

    for(handle=server->handle_db; handle!=NULL; handle=handle->next)
    {
        if(handle->fd == fd)
        {
            return handle;
        }
    }

    return NULL;
}
